package sun.management.snmp.jvminstr;

import java.lang.management.MemoryManagerMXBean;
import sun.management.snmp.util.SnmpCachedData;
import java.util.Map;
import java.lang.management.GarbageCollectorMXBean;
import sun.management.snmp.util.JvmContextFactory;
import com.sun.jmx.snmp.SnmpStatusException;
import com.sun.jmx.snmp.SnmpOid;
import sun.management.snmp.util.SnmpTableHandler;
import com.sun.jmx.snmp.agent.SnmpStandardObjectServer;
import com.sun.jmx.snmp.agent.SnmpMib;
import sun.management.snmp.util.MibLogger;
import sun.management.snmp.jvmmib.JvmMemGCTableMeta;

public class JvmMemGCTableMetaImpl extends JvmMemGCTableMeta
{
    static final long serialVersionUID = 8250461197108867607L;
    private transient JvmMemManagerTableMetaImpl managers;
    private static JvmMemGCTableMetaImpl.GCTableFilter filter;
    static final MibLogger log;
    
    public JvmMemGCTableMetaImpl(SnmpMib snmpMib, SnmpStandardObjectServer snmpStandardObjectServer) {
        super(snmpMib, snmpStandardObjectServer);
        this.managers = null;
    }
    
    private final JvmMemManagerTableMetaImpl getManagers(SnmpMib snmpMib) {
        if (this.managers == null) {
            this.managers = (JvmMemManagerTableMetaImpl)snmpMib.getRegisteredTableMeta("JvmMemManagerTable");
        }
        return this.managers;
    }
    
    protected SnmpTableHandler getHandler(Object o) {
        return this.getManagers(this.theMib).getHandler(o);
    }
    
    protected SnmpOid getNextOid(Object o) throws SnmpStatusException {
        return this.getNextOid(null, o);
    }
    
    protected SnmpOid getNextOid(SnmpOid obj, Object o) throws SnmpStatusException {
        int debugOn = JvmMemGCTableMetaImpl.log.isDebugOn();
        try {
            if (!(debugOn == 0)) {
                JvmMemGCTableMetaImpl.log.debug("getNextOid", "previous=" + obj);
            }
            SnmpTableHandler handler = this.getHandler(o);
            if (handler == null) {
                if (!(debugOn == 0)) {
                    JvmMemGCTableMetaImpl.log.debug("getNextOid", "handler is null!");
                }
                throw new SnmpStatusException(224);
            }
            else {
                SnmpOid next = JvmMemGCTableMetaImpl.filter.getNext(handler, obj);
                if (!(debugOn == 0)) {
                    JvmMemGCTableMetaImpl.log.debug("getNextOid", "next=" + next);
                }
                if (next == null) {
                    throw new SnmpStatusException(224);
                }
                return next;
            }
        }
        catch (RuntimeException ex) {
            if (!(debugOn == 0)) {
                JvmMemGCTableMetaImpl.log.debug("getNextOid", ex);
            }
            throw ex;
        }
    }
    
    protected boolean contains(SnmpOid snmpOid, Object o) {
        SnmpTableHandler handler = this.getHandler(o);
        return handler != null && JvmMemGCTableMetaImpl.filter.contains(handler, snmpOid);
    }
    
    public Object getEntry(SnmpOid snmpOid) throws SnmpStatusException {
        if (snmpOid == null) {
            throw new SnmpStatusException(224);
        }
        Map<String, JvmMemGCEntryImpl> userData = JvmContextFactory.getUserData();
        long oidArc = snmpOid.getOidArc(0);
        String s = (userData == null) ? null : ("JvmMemGCTable.entry." + oidArc);
        if (userData != null) {
            Object value = userData.get(s);
            if (value != null) {
                return value;
            }
        }
        SnmpTableHandler handler = this.getHandler(userData);
        if (handler == null) {
            throw new SnmpStatusException(224);
        }
        Object data = JvmMemGCTableMetaImpl.filter.getData(handler, snmpOid);
        if (data == null) {
            throw new SnmpStatusException(224);
        }
        JvmMemGCEntryImpl jvmMemGCEntryImpl = new JvmMemGCEntryImpl((GarbageCollectorMXBean)data, (int)oidArc);
        if (userData != null && jvmMemGCEntryImpl != null) {
            userData.put(s, jvmMemGCEntryImpl);
            return jvmMemGCEntryImpl;
        }
        return jvmMemGCEntryImpl;
    }
    
    static {
        JvmMemGCTableMetaImpl.filter = new JvmMemGCTableMetaImpl.GCTableFilter();
        log = new MibLogger(JvmMemGCTableMetaImpl.class);
    }
    
    protected static class GCTableFilter
    {
        public SnmpOid getNext(SnmpCachedData snmpCachedData, SnmpOid obj) {
            int debugOn = JvmMemGCTableMetaImpl.log.isDebugOn();
            int i = (obj == null) ? -1 : snmpCachedData.find(obj);
            if (!(debugOn == 0)) {
                JvmMemGCTableMetaImpl.log.debug("GCTableFilter", "oid=" + obj + " at insertion=" + i);
            }
            int j;
            if (i > -1) {
                j = i + 1;
            }
            else {
                j = -i - 1;
            }
            while (j < snmpCachedData.indexes.length) {
                if (!(debugOn == 0)) {
                    JvmMemGCTableMetaImpl.log.debug("GCTableFilter", "next=" + j);
                }
                Object o = snmpCachedData.datas[j];
                if (!(debugOn == 0)) {
                    JvmMemGCTableMetaImpl.log.debug("GCTableFilter", "value[" + j + "]=" + ((MemoryManagerMXBean)o).getName());
                }
                if ((o instanceof GarbageCollectorMXBean)) {
                    if (!(debugOn == 0)) {
                        JvmMemGCTableMetaImpl.log.debug("GCTableFilter", ((MemoryManagerMXBean)o).getName() + " is a  GarbageCollectorMXBean.");
                    }
                    return snmpCachedData.indexes[j];
                }
                else {
                    if (!(debugOn == 0)) {
                        JvmMemGCTableMetaImpl.log.debug("GCTableFilter", ((MemoryManagerMXBean)o).getName() + " is not a  GarbageCollectorMXBean: " + o.getClass().getName());
                    }
                    ++j;
                }
            }
            return null;
        }
        
        public SnmpOid getNext(SnmpTableHandler snmpTableHandler, SnmpOid next) {
            if ((snmpTableHandler instanceof SnmpCachedData)) {
                return this.getNext((SnmpCachedData)snmpTableHandler, next);
            }
            do {
                next = snmpTableHandler.getNext(next);
                if ((snmpTableHandler.getData(next) instanceof GarbageCollectorMXBean)) {
                    return next;
                }
                continue;
            } while (next != null);
            return null;
        }
        
        public Object getData(SnmpTableHandler snmpTableHandler, SnmpOid snmpOid) {
            Object data = snmpTableHandler.getData(snmpOid);
            if ((data instanceof GarbageCollectorMXBean)) {
                return data;
            }
            return null;
        }
        
        public boolean contains(SnmpTableHandler snmpTableHandler, SnmpOid snmpOid) {
            return snmpTableHandler.getData(snmpOid) instanceof GarbageCollectorMXBean;
        }
    }
}
